### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages
import codecs
import string

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages

# import local packages


_unicodeTransTable = {}
def _fillUnicodeTransTable():
    _corresp = [
        (u"A", [0x00C0, 0x00C1, 0x00C2, 0x00C3, 0x00C4, 0x00C5, 0x0100, 0x0102, 0x0104]),
        (u"AE", [0x00C6]),
        (u"a", [0x00E0, 0x00E1, 0x00E2, 0x00E3, 0x00E4, 0x00E5, 0x0101, 0x0103, 0x0105]),
        (u"ae", [0x00E6]),
        (u"C", [0x00C7, 0x0106, 0x0108, 0x010A, 0x010C]),
        (u"c", [0x00E7, 0x0107, 0x0109, 0x010B, 0x010D]),
        (u"D", [0x00D0, 0x010E, 0x0110]),
        (u"d", [0x00F0, 0x010F, 0x0111]),
        (u"E", [0x00C8, 0x00C9, 0x00CA, 0x00CB, 0x0112, 0x0114, 0x0116, 0x0118, 0x011A]),
        (u"e", [0x00E8, 0x00E9, 0x00EA, 0x00EB, 0x0113, 0x0115, 0x0117, 0x0119, 0x011B]),
        (u"G", [0x011C, 0x011E, 0x0120, 0x0122]),
        (u"g", [0x011D, 0x011F, 0x0121, 0x0123]),
        (u"H", [0x0124, 0x0126]),
        (u"h", [0x0125, 0x0127]),
        (u"I", [0x00CC, 0x00CD, 0x00CE, 0x00CF, 0x0128, 0x012A, 0x012C, 0x012E, 0x0130]),
        (u"i", [0x00EC, 0x00ED, 0x00EE, 0x00EF, 0x0129, 0x012B, 0x012D, 0x012F, 0x0131]),
        (u"IJ", [0x0132]),
        (u"ij", [0x0133]),
        (u"J", [0x0134]),
        (u"j", [0x0135]),
        (u"K", [0x0136]),
        (u"k", [0x0137, 0x0138]),
        (u"L", [0x0139, 0x013B, 0x013D, 0x013F, 0x0141]),
        (u"l", [0x013A, 0x013C, 0x013E, 0x0140, 0x0142]),
        (u"N", [0x00D1, 0x0143, 0x0145, 0x0147, 0x014A]),
        (u"n", [0x00F1, 0x0144, 0x0146, 0x0148, 0x0149, 0x014B]),
        (u"O", [0x00D2, 0x00D3, 0x00D4, 0x00D5, 0x00D6, 0x00D8, 0x014C, 0x014E, 0x0150]),
        (u"o", [0x00F2, 0x00F3, 0x00F4, 0x00F5, 0x00F6, 0x00F8, 0x014D, 0x014F, 0x0151]),
        (u"OE", [0x0152]),
        (u"oe", [0x0153]),
        (u"R", [0x0154, 0x0156, 0x0158]),
        (u"r", [0x0155, 0x0157, 0x0159]),
        (u"S", [0x015A, 0x015C, 0x015E, 0x0160]),
        (u"s", [0x015B, 0x015D, 0x015F, 0x01610, 0x017F]),
        (u"T", [0x0162, 0x0164, 0x0166]),
        (u"t", [0x0163, 0x0165, 0x0167]),
        (u"U", [0x00D9, 0x00DA, 0x00DB, 0x00DC, 0x0168, 0x016A, 0x016C, 0x016E, 0x0170, 0x172]),
        (u"u", [0x00F9, 0x00FA, 0x00FB, 0x00FC, 0x0169, 0x016B, 0x016D, 0x016F, 0x0171]),
        (u"W", [0x0174]),
        (u"w", [0x0175]),
        (u"Y", [0x00DD, 0x0176, 0x0178]),
        (u"y", [0x00FD, 0x00FF, 0x0177]),
        (u"Z", [0x0179, 0x017B, 0x017D]),
        (u"z", [0x017A, 0x017C, 0x017E])
        ]
    for char, codes in _corresp:
        for code in codes :
            _unicodeTransTable[code] = char

_fillUnicodeTransTable()


def translateString(s, escapeSlashes=False, forceLower=True, spaces=' ') :
    """Remove extended characters from string and replace them with 'basic' ones
    
    @param s: text to be cleaned.
    @type s: str or unicode
    @param escapeSlashes: if True, slashes are also converted
    @type escapeSlashes: boolean
    @param forceLower: if True, result is automatically converted to lower case
    @type forceLower: boolean
    @return: text without diacritics
    @rtype: unicode
    """
    if escapeSlashes:
        s = string.replace(s, "\\", "/").split("/")[-1]
    s = s.strip()
    if isinstance(s, str):
        s = unicode(s, "utf8", "replace")
    s = s.translate(_unicodeTransTable)
    s = ''.join([a for a in s.translate(_unicodeTransTable) if a.replace(' ', '-') in (string.ascii_letters + string.digits + '_-.')])
    if forceLower:
        s = s.lower()
    if spaces != ' ':
        s = s.replace(' ', spaces)
    return s


def nvl(value, default=''):
    """Get specified value, or an empty string if value is empty
    
    @param value: text to be checked
    @param default: default value
    @return: value, or default if value is empty
    """
    return value or default


def uninvl(value, default=u''):
    """Get specified value converted to unicode, or an empty unicode string if value is empty
    
    @param value: text to be checked
    @type value: str or unicode
    @param default: default value
    @return: value, or default if value is empty
    @rtype: unicode
    """
    try:
        if isinstance(value, unicode):
            return value
        return codecs.decode(value or default)
    except:
        return codecs.decode(value or default, 'latin1')


def unidict(value):
    """Get specified dict with values converted to unicode
    
    @param value: input dict of strings which may be converted to unicode
    @type value: dict
    @return: input dict converted to unicode
    @rtype: dict
    """
    result = {}
    for key in value:
        result[key] = uninvl(value[key])
    return result


def unilist(value):
    """Get specified list with values converted to unicode
    
    @param value: input list of strings which may be converted to unicode
    @type value: list
    @return: input list converted to unicode
    @rtype: list
    """
    if not isinstance(value, (list, tuple)):
        return uninvl(value)
    return [uninvl(v) for v in value]


def utf8(value):
    """Convert given value to UTF-8"""
    if isinstance(value, unicode):
        value = value.encode('utf8')
    return value



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages
import re
from persistent import Persistent
from BTrees import IFBTree

# import Zope3 interfaces
from zope.index.interfaces import IInjection, IStatistics, IIndexSearch
from zopyx.txng3.core.interfaces import IStorageWithTermFrequency
from zopyx.txng3.core.interfaces.ting import ITingIndex

# import local interfaces

# import Zope3 packages
from zope.catalog.attribute import AttributeIndex
from zope.component import createObject
from zope.container.contained import Contained
from zope.interface import implements
from zopyx.txng3.core import config
from zopyx.txng3.core.index import Index

# import local packages
from hurry.query.query import IndexTerm


class TextIndexNG(AttributeIndex, Persistent, Contained):
    """Adaptation of zopyx.txng3.core for use zope.catalog index"""

    implements(IInjection, IStatistics, IIndexSearch, ITingIndex)

    def __init__(self,
                 field_name=None,
                 interface=None,
                 field_callable=False,
                 use_stemmer=config.defaults['use_stemmer'],
                 dedicated_storage=config.defaults['dedicated_storage'],
                 ranking=config.defaults['ranking'],
                 use_normalizer=config.defaults['use_normalizer'],
                 languages=config.DEFAULT_LANGUAGE,
                 use_stopwords=config.defaults['use_stopwords'],
                 autoexpand_limit=config.defaults['autoexpand_limit'],
                 splitter=config.DEFAULT_SPLITTER,
                 index_unknown_languages=config.defaults['index_unknown_languages'],
                 query_parser=config.DEFAULT_PARSER,
                 lexicon=config.DEFAULT_LEXICON,
                 splitter_additional_chars=config.defaults['splitter_additional_chars'],
                 storage=config.DEFAULT_STORAGE,
                 splitter_casefolding=config.defaults['splitter_casefolding']):
        spaces = re.compile(r'\s+')
        if ranking:
            util = createObject(storage)
            if not IStorageWithTermFrequency.providedBy(util):
                raise ValueError("This storage cannot be used for ranking")
        _fields = spaces.split(field_name)
        AttributeIndex.__init__(self, _fields[0], interface, field_callable)
        if len(_fields) < 2:
            dedicated_storage = False
        self._index = Index(fields=_fields,
                            languages=spaces.split(languages),
                            use_stemmer=use_stemmer,
                            dedicated_storage=dedicated_storage,
                            ranking=ranking,
                            use_normalizer=use_normalizer,
                            use_stopwords=use_stopwords,
                            storage=storage,
                            autoexpand_limit=autoexpand_limit,
                            splitter=splitter,
                            lexicon=lexicon,
                            index_unknown_languages=index_unknown_languages,
                            query_parser=query_parser,
                            splitter_additional_chars=splitter_additional_chars,
                            splitter_casefolding=splitter_casefolding)
        self.languages = languages
        self.use_stemmer = use_stemmer
        self.dedicated_storage = dedicated_storage
        self.ranking = ranking
        self.use_normalizer = use_normalizer
        self.use_stopwords = use_stopwords
        self.interface = interface
        self.storage = storage
        self.autoexpand_limit = autoexpand_limit
        self.default_field = _fields[0]
        self._fields = _fields
        self.splitter = splitter
        self.lexicon = lexicon
        self.index_unknown_languages = index_unknown_languages
        self.query_parser = query_parser
        self.splitter_additional_chars = splitter_additional_chars
        self.splitter_casefolding = splitter_casefolding

    def clear(self):
        self._index.clear()

    def documentCount(self):
        """See interface IStatistics"""
        return len(self._index.getStorage(self.default_field))

    def wordCount(self):
        """See interface IStatistics"""
        return len(self._index.getLexicon())

    def index_doc(self, docid, value):
        """See interface IInjection"""
        v = self.interface(value, None)
        if v is not None:
            self.unindex_doc(docid)
            self._index.index_object(v, docid)

    def unindex_doc(self, docid):
        """See interface IInjection"""
        self._index.unindex_object(docid)

    def apply(self, query):
        if isinstance(query, dict):
            kw = query
            query = kw['query']
            del kw['query']
        ting_rr = self._index.search(query, **kw)
        return ting_rr.getDocids().keys()


class Text(IndexTerm):
    """hurry.query search term"""

    def __init__(self, index_id, text):
        super(Text, self).__init__(index_id)
        self.text = text

    def getIndex(self, context=None):
        index = super(Text, self).getIndex(context)
        assert ITingIndex.providedBy(index)
        return index

    def apply(self, context=None):
        index = self.getIndex(context)
        return IFBTree.IFSet(index.apply(self.text))



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.tales.interfaces import ITALESFunctionNamespace

# import local interfaces
from ztfy.utils.tal.interfaces import IRequestDataTalesAPI

# import Zope3 packages
from zope.interface import implements

# import local packages
from ztfy.utils.request import getData


class RequestDataTalesAdapter(object):

    implements(IRequestDataTalesAPI, ITALESFunctionNamespace)

    def __init__(self, context):
        self.context = context

    def setEngine(self, engine):
        self.request = engine.vars['request']

    def __getattr__(self, attr):
        return getData(attr, self.context)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages

# import Zope3 interfaces
from zope.publisher.interfaces import IRequest
from zope.annotation.interfaces import IAnnotations

# import local interfaces

# import Zope3 packages
from zope.security.management import getInteraction

# import local packages

from ztfy.utils import _
from zope.security.interfaces import NoInteraction


def getRequest():
    """Extract request from current interaction"""
    interaction = getInteraction()
    for participation in interaction.participations:
        if IRequest.providedBy(participation):
            return participation
    raise RuntimeError, _("No Request in interaction !")


def queryRequest():
    try:
        return getRequest()
    except NoInteraction:
        return None


def getRequestPrincipal(request=None):
    """Get principal from given request"""
    if request is None:
        request = getRequest()
    return request.principal

getPrincipal = getRequestPrincipal


def getRequestAnnotations(request=None):
    """Get annotations from given request"""
    if request is None:
        request = getRequest()
    return IAnnotations(request)

getAnnotations = getRequestAnnotations


def getRequestData(key, request=None, default=None):
    """Get request data, stored into request annotations"""
    try:
        annotations = getRequestAnnotations(request)
        return annotations.get(key, default)
    except:
        return default

getData = getRequestData


def setRequestData(key, data, request=None):
    """Define request data, stored into request annotations"""
    annotations = getRequestAnnotations(request)
    annotations[key] = data

setData = setRequestData



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages
from sgmllib import SGMLParser

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages

# import local packages


class HTMLParser(SGMLParser):

    data = ''
    entitydefs = { 'amp': '&', 'lt': '<', 'gt': '>',
                   'apos': "'", 'quot': '"',
                   'Agrave': 'À', 'Aacute': 'A', 'Acirc': 'Â', 'Atilde': 'A', 'Auml': 'Ä', 'Aring': 'A',
                   'AElig': 'AE',
                   'Ccedil': 'Ç',
                   'Egrave': 'É', 'Eacute': 'È', 'Ecirc': 'Ê', 'Euml': 'Ë',
                   'Igrave': 'I', 'Iacute': 'I', 'Icirc': 'I', 'Iuml': 'I',
                   'Ntilde': 'N',
                   'Ograve': 'O', 'Oacute': 'O', 'Ocirc': 'Ô', 'Otilde': 'O', 'Ouml': 'Ö', 'Oslash': 'O',
                   'Ugrave': 'Ù', 'Uacute': 'U', 'Ucirc': 'Û', 'Uuml': 'Ü',
                   'Yacute': 'Y',
                   'THORN': 'T',
                   'agrave': 'à', 'aacute': 'a', 'acirc': 'â', 'atilde': 'a', 'auml': 'ä', 'aring': 'a', 'aelig': 'ae',
                   'ccedil': 'ç',
                   'egrave': 'è', 'eacute': 'é', 'ecirc': 'ê', 'euml': 'ë',
                   'igrave': 'i', 'iacute': 'i', 'icirc': 'î', 'iuml': 'ï',
                   'ntilde': 'n',
                   'ograve': 'o', 'oacute': 'o', 'ocirc': 'ô', 'otilde': 'o', 'ouml': 'ö', 'oslash': 'o',
                   'ugrave': 'ù', 'uacute': 'u', 'ucirc': 'û', 'uuml': 'ü',
                   'yacute': 'y',
                   'thorn': 't',
                   'yuml': 'ÿ' }

    charrefs = {  34 : '"', 38 : '&', 39 : "'",
                  60 : '<', 62 : '>',
                 192 : 'À', 193 : 'A', 194 : 'Â', 195 : 'A', 196 : 'Ä', 197 : 'A',
                 198 : 'AE',
                 199 : 'Ç',
                 200 : 'È', 201 : 'É', 202 : 'Ê', 203 : 'Ë',
                 204 : 'I', 205 : 'I', 206 : 'Î', 207 : 'Ï',
                 208 : 'D',
                 209 : 'N',
                 210 : 'O', 211 : 'O', 212 : 'Ô', 213 : 'O', 214 : 'Ö', 216 : 'O',
                 215 : 'x',
                 217 : 'Ù', 218 : 'U', 219 : 'Û', 220 : 'Ü',
                 221 : 'Y', 222 : 'T',
                 223 : 'sz',
                 224 : 'à', 225 : 'a', 226 : 'â', 227 : 'a', 228 : 'ä', 229 : 'a',
                 230 : 'ae',
                 231 : 'ç',
                 232 : 'è', 233 : 'é', 234 : 'ê', 235 : 'ë',
                 236 : 'i', 237 : 'i', 238 : 'î', 239 : 'ï',
                 240 : 'e',
                 241 : 'n',
                 242 : 'o', 243 : 'o', 244 : 'ô', 245 : 'o', 246 : 'ö', 248 : 'o',
                 249 : 'ù', 250 : 'u', 251 : 'û', 252 : 'ü',
                 253 : 'y', 255 : 'ÿ' }

    def handle_data(self, data):
        try:
            self.data += data
        except:
            self.data += unicode(data, 'utf8')

    def handle_charref(self, name):
        try:
            n = int(name)
        except ValueError:
            self.unknown_charref(name)
            return
        if not 0 <= n <= 255:
            self.unknown_charref(name)
            return
        self.handle_data(self.charrefs.get(n) or unicode(chr(n), 'latin1'))

    def start_td(self, attributes):
        self.data += ' '

    def start_p(self, attributes):
        pass

    def end_p(self):
        self.data += '\n'


def htmlToText(value):
    """Utility function to extract text content from HTML"""
    if value is None:
        return ''
    parser = HTMLParser()
    parser.feed(value)
    parser.close()
    return parser.data



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008-2010 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

__docformat__ = "restructuredtext"

# import standard packages

# import Zope3 interfaces
from zope.component.interfaces import IObjectEvent

# import local interfaces

# import Zope3 packages

# import local packages


class INewSiteManagerEvent(IObjectEvent):
    """Event interface for new site manager event"""



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages
from persistent.list import PersistentList
from persistent.dict import PersistentDict

# import Zope3 interfaces
from zope.authentication.interfaces import IAuthentication
from zope.pluggableauth.interfaces import IPrincipalInfo

# import local interfaces

# import Zope3 packages
from zc.set import Set
from zope.component import getUtility
from zope.deprecation.deprecation import deprecate
from zope.interface import implements
from zope.security.proxy import removeSecurityProxy

# import local packages


def unproxied(value):
    """Remove security proxies from given value ; if value is a list or dict, all elements are unproxied"""
    if isinstance(value, (list, PersistentList)):
        result = []
        for v in value:
            result.append(unproxied(v))
    elif isinstance(value, (dict, PersistentDict)):
        result = value.copy()
        for v in value:
            result[v] = unproxied(value[v])
    elif isinstance(value, (set, Set)):
        result = []
        for v in value:
            result.append(unproxied(v))
    else:
        result = removeSecurityProxy(value)
    return result


@deprecate("ztfy.utils.security.MissingPrincipal is deprecated. Use ztfy.security.search.MissingPrincipal class instead.")
class MissingPrincipal(object):

    implements(IPrincipalInfo)

    def __init__(self, id):
        self.id = id

    @property
    def title(self):
        return _("< missing principal %s>") % self.id

    @property
    def description(self):
        return _("This principal can't be found in any authentication utility...")


@deprecate("ztfy.utils.security.getPrincipal is deprecated. Use ztfy.security.search.getPrincipal function instead.")
def getPrincipal(uid):
    principals = getUtility(IAuthentication)
    try:
        return principals.getPrincipal(uid)
    except:
        return MissingPrincipal(uid)



### -*- coding: utf-8 -*- ####################################################
##############################################################################
#
# Copyright (c) 2008 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################


# import standard packages
from datetime import datetime

# import Zope3 interfaces

# import local interfaces

# import Zope3 packages
from zope.datetime import parseDatetimetz
from zope.i18n import translate

# import local packages
from timezone import gmtime

from ztfy.utils import _


def unidate(value):
    """Get specified date converted to unicode ISO format
    
    Dates are always assumed to be stored in GMT timezone
    
    @param value: input date to convert to unicode
    @type value: date or datetime
    @return: input date converted to unicode
    @rtype: unicode
    """
    if value is not None:
        value = gmtime(value)
        return unicode(value.isoformat('T'), 'ascii')
    return None


def parsedate(value):
    """Get date specified in unicode ISO format to Python datetime object
    
    Dates are always assumed to be stored in GMT timezone
    
    @param value: unicode date to be parsed
    @type value: unicode
    @return: the specified value, converted to datetime
    @rtype: datetime
    """
    if value is not None:
        return gmtime(parseDatetimetz(value))
    return None


def datetodatetime(value):
    """Get datetime value converted from a date or datetime object
    
    @param value: a date or datetime value to convert
    @type value: date or datetime
    @return: input value converted to datetime
    @rtype: datetime
    """
    if type(value) is datetime:
        return value
    return datetime(value.year, value.month, value.day)


def getAge(value):
    """Get age of a given datetime (including timezone) compared to current datetime (in UTC)
    
    @param value: a datetime value, including timezone
    @type value: datetime
    @return: string representing value age
    @rtype: gettext translated string
    """
    now = gmtime(datetime.utcnow())
    delta = now - value
    if delta.days > 60:
        return translate(_("%d months ago")) % int(round(delta.days * 1.0 / 30))
    elif delta.days > 10:
        return translate(_("%d weeks ago")) % int(round(delta.days * 1.0 / 7))
    elif delta.days > 2:
        return translate(_("%d days ago")) % delta.days
    elif delta.days == 2:
        return translate(_("the day before yesterday"))
    elif delta.days == 1:
        return translate(_("yesterday"))
    else:
        hours = int(round(delta.seconds * 1.0 / 3600))
        if hours > 1:
            return translate(_("%d hours ago")) % hours
        elif delta.seconds > 300:
            return translate(_("%d minutes ago")) % int(round(delta.seconds * 1.0 / 60))
        else:
            return translate(_("less than 5 minutes ago"))
